indexing
	description: "[
					A closeup dialog is composed of a GUI_SPECIAL_FRAME. Inside the
					frame an image, a headline and various option texts can be
					displayed. The user can select one of the options. You can
					specify an event action which is executed when the user	
					selects an option.
					
					This class is part of the XAE Extended Adventure Engine Project.
				  ]"

	author: "Ralph Wiedemeier, ralphw@student.ethz.ch"
	date: "$Date: 2007-02-05 18:24:28 +0100 (Mon, 05 Feb 2007) $"
	revision: "$Revision: 2485 $"

class
	GUI_CLOSEUP

inherit
	GUI_SPECIAL_FRAME
	rename
		draw as draw_frame
	redefine
		publish_event_click,
		publish_event_hoover
	end

	GUI_SHARED
		export {NONE} all
		undefine out end

create
	make_from_game_closeup

feature -- Initialization

	make_from_game_closeup (a_closeup: CLOSEUP) is
			-- Create new closeup dialog from game closeup
		require
			valid_closeup: a_closeup /= Void
		do
			create_frame
			create_image (a_closeup.image_path)
			create_head (a_closeup.description)
			from
				a_closeup.options.start
			until
				a_closeup.options.after
			loop
				create_option (a_closeup.options.item.text, a_closeup.options.item)
				a_closeup.options.forth
			end
		end

feature -- Status

	set_max_characters (a_number: INTEGER) is
			-- Set maximum number of displayed characters
		do
			if a_number >= 0 then
				max_characters := a_number
			else
				max_characters := 0
			end
		end

	max_characters: INTEGER

	set_event_action_option_click (an_action: PROCEDURE [ANY, TUPLE]) is
			-- Register action for clicks on options
		do
			action_option_click := an_action
		end


feature -- Operations

	draw (a_surface: EM_VIDEO_SURFACE) is
			-- Draw representation of current to 'a_surface'
		local
			n: INTEGER
		do
			n := max_characters

			draw_frame (a_surface)
			image.draw (a_surface)
			head.font.set_max_characters (n)
			head.draw (a_surface)
			n := n - head.text.count

			from
				options.start
			until
				options.after
			loop
				options.item.font.set_max_characters (n)
				options.item.draw (a_surface)
				n := n - options.item.text.count
				options.forth
			end
		end

	dispose_texture_memory is
			-- Free memory of used textures
		do
			image.dispose_texture_memory
		end

feature {GUI_SPECIAL_FRAME} -- Event handling

	handle_option_click_event (a_target: GUI_SPECIAL_FRAME) is
			-- Handle mouse clicks on options
		do
			action_option_click.call ([a_target])
		end

feature -- Event publishing

	publish_event_click (mouse_x: INTEGER; mouse_y: INTEGER) is
			-- publish click event to closeup frame and to all options
		do
			Precursor (mouse_x, mouse_y)

			from
				options.start
			until
				options.after
			loop
				options.item.publish_event_click (mouse_x, mouse_y)
				options.forth
			end
		end

	publish_event_hoover (mouse_x: INTEGER; mouse_y: INTEGER) is
			-- publish hoover event to option frame and to all options
		do
			Precursor (mouse_x, mouse_y)

			from
				options.start
			until
				options.after
			loop
				options.item.publish_event_hoover (mouse_x, mouse_y)
				options.forth
			end
		end

feature {NONE} -- Internal routines

	create_frame is
			-- Initialization
		do
			-- Set parameters for special frame
			make_from_position_and_size (closeup_x, closeup_y, closeup_width, closeup_height)
			set_special_corner_top_right
			set_special_corner_size (25)
			set_fill_color_normal (gui_constants.color_closeup_fill)
			set_border_color_normal (gui_constants.color_closeup_border_normal)
			set_border_color_hoover (gui_constants.color_closeup_border_hoover)

			-- Set position offsets
			text_x := x + height + 20
			text_width := width - height - 20

			-- Create option list
			create options.make
		end

	create_image (a_path: STRING) is
			-- create image of closeup
		local
			img_x, img_y: INTEGER
		do
			create image.make_with_image (a_path)
			image.set_layout_vertical
			image.auto_resize_height
			image.auto_resize_width

			 img_x := closeup_x + padding + (image_max_width - image.width) // 2
			 img_y := closeup_y + (closeup_height - image.height) // 2



			image.set_position (img_x, img_y)

			image.set_border_thickness (0)
			image.set_fill_color_normal (gui_constants.color_transparent)
		end

	create_head (a_text: STRING) is
			-- create headline of closeup
		do
			create head.make_with_text (a_text, gui_constants.font_normal)
			head.set_layout_horizontal
			head.set_font_size (1.0)
			head.set_padding (8)

			text_x := closeup_x + image_max_width + 2 * padding
			text_y := closeup_y + padding
			text_width := closeup_width - image_max_width - 3 * padding

			head.set_position (text_x, text_y)
			head.set_size (text_width, 0)
			head.enable_multiline
			head.auto_resize_height

			head.set_border_thickness (1)
			head.set_border_color_normal (gui_constants.color_closeup_border_normal)
			head.set_special_corner_top_right
			head.set_special_corner_size (15)
			head.set_fill_color_normal (gui_constants.color_dark_grey )

			text_y := text_y + head.height + padding
		end

	create_option (a_text: STRING; an_object: ANY) is
			-- Set common parameters for option text buttons
		local
			opt: GUI_SPECIAL_BUTTON
		do
			create opt.make_with_text (a_text, gui_constants.font_normal)
			opt.set_data (an_object)
			opt.set_layout_horizontal
			opt.set_font_size (0.9)
			opt.set_padding (8)

			opt.set_position (text_x, text_y)
			opt.set_size (text_width, 0)

			opt.enable_multiline
			opt.auto_resize_height

			opt.set_border_thickness (0)
			opt.set_fill_color_normal (gui_constants.color_transparent)
			opt.set_fill_color_hoover (gui_constants.color_option_fill_hoover)

			opt.set_event_action_click (agent handle_option_click_event)

			options.extend (opt)
			text_y := text_y + opt.height + padding // 2
		end

feature -- Internal variables

	head: GUI_SPECIAL_BUTTON
	image: GUI_SPECIAL_BUTTON
	options: LINKED_LIST [GUI_SPECIAL_BUTTON]

	action_option_click: PROCEDURE [ANY, TUPLE]

	closeup_x: INTEGER is 80
	closeup_y: INTEGER is 115
	closeup_width: INTEGER is 864
	closeup_height: INTEGER is 435
	image_max_width: INTEGER is 300

	padding: INTEGER is 17

	text_x: INTEGER
	text_y: INTEGER
	text_width: INTEGER

end -- class GUI_CLOSEUP
