indexing
	description: "[
					Each scene object contains all information for one adventure
					location including cylindrical background pane, static and
					moving foreground objects and clickable areas.
					
					The scene is able to draw itself including the overlying
					screen mask, item repository and main menu. It handles mouse
					events and feeds them to the appropriate objects.
					
					This class is part of the XAE Extended Adventure Engine Project.
				  ]"

	author: "Ralph Wiedemeier, ralphw@student.ethz.ch"
	date: "$Date: 2007-02-05 18:24:28 +0100 (Mon, 05 Feb 2007) $"
	revision: "$Revision: 2485 $"

class
	GUI_GAME_SCENE

inherit
	GUI_SCENE
	redefine
		initialize_scene,
		handle_option_click_event,
		handle_clickable_click_event,
		handle_menu_event_open_script,
		handle_menu_event_load_game,
		handle_menu_event_save_game,
		handle_menu_event_credits,
		handle_timer_event
	end

create
	make_from_game_scene

feature -- Initialization

	make_from_game_scene (a_scene: SCENE) is
			-- Create new displayable scene from game scene object
		local
			rect: EM_RECT
			item: GUI_CLICKABLE_3D
		do
			default_create
			current_game_scene := a_scene

			-- Create clickables list
			create clickables.make

			-- Attach timer list
			timers := a_scene.timers.twin

			-- Set panoramic background images	
			first_background_image := a_scene.background_path
			second_background_image := a_scene.background_path_2

			-- Create clickable areas
			from
				a_scene.areas.start
			until
				a_scene.areas.after
			loop
				create rect.make (a_scene.areas.item.x,
								  a_scene.areas.item.y,
								  a_scene.areas.item.width,
								  a_scene.areas.item.height)

				create item.make_from_rect (rect)
				item.set_description (a_scene.areas.item.description)
				item.set_data (a_scene.areas.item)
				item.set_event_action_click (agent handle_clickable_click_event)
				item.set_event_action_hoover_in (agent handle_clickable_hoover_in_event)
				item.set_event_action_hoover_out (agent handle_clickable_hoover_out_event)
				clickables.extend (item)

				a_scene.areas.forth
			end

			-- Create clickable items
			from
				a_scene.items.start
			until
				a_scene.items.after
			loop
				if a_scene.items.item.is_visible then
					if a_scene.items.item.animation /= Void then
						create item.make_animated (a_scene.items.item.default_path,
												    a_scene.items.item.x, a_scene.items.item.y,
												    a_scene.items.item.animation)
					else
						create item.make_from_position_and_image (a_scene.items.item.default_path,
															  	   a_scene.items.item.x, a_scene.items.item.y)
					end
					item.set_visible
					item.set_description (a_scene.items.item.description)
					item.set_data (a_scene.items.item)
					item.set_event_action_click (agent handle_clickable_click_event)
					item.set_event_action_hoover_in (agent handle_clickable_hoover_in_event)
					item.set_event_action_hoover_out (agent handle_clickable_hoover_out_event)
					clickables.extend (item)
				end
				a_scene.items.forth
			end

			-- Update repository
			game_skin.repository.update (gui.engine.get_repository)

			-- Set start angle for scene
			angle := a_scene.angle
		end

	initialize_scene is
			-- Do game scene specific initialization
		do
			Precursor
			game_skin.main_menu.enable_all
		end


feature {GUI_CLICKABLE} -- Event handlers

	handle_clickable_click_event (clickable: GUI_CLICKABLE_3D) is
			-- This feature is called when an item in the current game scene is clicked
		local
			target: TRIGGER
			event: EVENT
			context: ITEM
		do
			target ?= clickable.data
			if target /= Void then
				if game_skin.repository.selected_item /= Void then
					context ?= game_skin.repository.selected_item.data
					game_skin.repository.deselect
				end
				create event.make (target, context)
				process_game_event (event)
			end
		end

	handle_clickable_hoover_in_event (clickable: GUI_CLICKABLE_3D) is
			-- This feature is called when the mouse hoovers into a clickable item
		do
			tooltip.set_text (clickable.out)
			tooltip.auto_resize_height
			tooltip.set_visible
		end

	handle_clickable_hoover_out_event (clickable: GUI_CLICKABLE_3D) is
			-- This feature is called when the mouse hoovers out of a clickable item
		do
			tooltip.set_invisible
		end

	handle_option_click_event (clickable: GUI_CLICKABLE) is
			-- This feature is called when an option in a closeup dialog is clicked
		local
			target: OPTION
			event: EVENT
			context: ITEM
		do
			target ?= clickable.data
			if target /= Void then
				if game_skin.repository.selected_item /= Void then
					context ?= game_skin.repository.selected_item.data
					game_skin.repository.deselect
				end
				create event.make (target, context)
				process_game_event (event)
			end
		end

	handle_menu_event_open_script is
			-- Button 'open script' is selected from main menu
		do
			if drawing_mode = drawing_mode_closeup then
				game_skin.main_menu.enable_all
				prepare_for_scene_view
			else
				closeup := create {GUI_FILE_DIALOG}.make_script
				from
					closeup.options.start
				until
					closeup.options.after
				loop
					closeup.options.item.set_event_action_click
						(agent script_close_closeup (closeup.options.item.data))
					closeup.options.forth
				end
				game_skin.main_menu.disable_except (game_skin.main_menu.button_open_script)
				prepare_for_closeup_view
			end
		end

	handle_menu_event_load_game is
			-- Button 'load game' is selected from main menu
		do
			if drawing_mode = drawing_mode_closeup then
				game_skin.main_menu.enable_all
				prepare_for_scene_view
			else
				closeup := create {GUI_FILE_DIALOG}.make_load
				from
					closeup.options.start
				until
					closeup.options.after
				loop
					closeup.options.item.set_event_action_click
						(agent load_close_closeup (closeup.options.item.data))
					closeup.options.forth
				end
				game_skin.main_menu.disable_except (game_skin.main_menu.button_load_game)
				prepare_for_closeup_view
			end
		end

	handle_menu_event_save_game is
			-- Button 'save game' is selected from main menu
		do
			if drawing_mode = drawing_mode_closeup then
				game_skin.main_menu.enable_all
				prepare_for_scene_view
			else
				closeup := create {GUI_FILE_DIALOG}.make_save
				from
					closeup.options.start
				until
					closeup.options.after
				loop
					closeup.options.item.set_event_action_click
						(agent save_close_closeup (closeup.options.item.data))
					closeup.options.forth
				end
				game_skin.main_menu.disable_except (game_skin.main_menu.button_save_game)
				prepare_for_closeup_view
			end
		end

	handle_menu_event_credits is
			-- Button 'credits' is selected from main menu
		local
			transition: GUI_TRANSITION_DISSOLVE
			credits: GUI_CREDITS
		do
			prepare_scene_transition
			create credits.make
			credits.set_previous_game_scene (current_game_scene)
			next_scene := credits
			next_scene.initialize_scene

			create transition
			transition.do_transition (Current, next_scene, screen)
		end

	handle_timer_event (a_time: INTEGER) is
			-- Check scene timers and act appropriately
		local
			event: EVENT
		do
			from
				timers.start
			until
				timers.after
			loop
				if a_time > timers.item.timeout then
					create event.make (timers.item, Void)
					process_game_event (event)
					timers.remove
				else
					timers.forth
				end
			end
		end


feature {NONE} -- Implementation

	process_game_event (an_event: EVENT) is
			-- Send 'an_event' to the engine and establish new game state
		do
			if an_event /= Void then
				gui.engine.event_occured (an_event)

				-- Check for scene change
				if gui.engine.scene_changed then
					do_scene_change
				-- Check for picked items
				elseif gui.engine.items_picked_up then
					do_clickables_update
				end

				-- Check if a close up should be displayed
				if drawing_mode = drawing_mode_closeup
						and then gui.engine.get_closeup = Void then
					if closeup /= Void then
						closeup.dispose_texture_memory
						closeup := Void
					end
					prepare_for_scene_view

				elseif gui.engine.get_closeup /= Void then
					prepare_for_closeup_view
					current_game_closeup := gui.engine.get_closeup
					if closeup /= Void then
						closeup.dispose_texture_memory
						closeup := Void
					end
					show_game_closeup (current_game_closeup)
				end

				-- Check if repository has changed
				if gui.engine.repository_changed then
					game_skin.repository.update (gui.engine.get_repository)
				end

				-- Get status message
				game_skin.set_status_message (gui.engine.get_message)
			end
		end

	do_scene_change is
			-- Load new scene using the appropriate transition effect
		local
			transition_name: STRING
			transition: GUI_TRANSITION
		do
			if gui.engine.get_current_scene /= current_game_scene then
				prepare_scene_transition
				next_scene := create {GUI_GAME_SCENE}.make_from_game_scene (gui.engine.get_current_scene)
				next_scene.initialize_scene

				transition_name := gui.engine.get_current_scene.transition
				if transition_name = Void then
					transition_name := "zoom"
				end
				if transition_name.is_equal ("dissolve") then
					transition := create {GUI_TRANSITION_DISSOLVE}
				elseif transition_name.is_equal ("cube") then
					transition := create {GUI_TRANSITION_CUBE}
				elseif transition_name.is_equal ("explosion") then
					transition := create {GUI_TRANSITION_EXPLODE}
				else
					transition := create {GUI_TRANSITION_ZOOM}
					transition.set_center (mouse_click_x, mouse_click_y)
				end
				if not unrecoverable_error then
					transition.do_transition (Current, next_scene, screen)
				end
			end
		end

	do_clickables_update is
			-- Update visibility state of all clickables
		local
			item: ITEM
		do
			from
				clickables.start
			until
				clickables.after
			loop
				item ?= clickables.item.data
				if item /= Void and then not item.is_visible then
					clickables.item.set_invisible
					clickables.item.set_disabled
					tooltip.set_invisible
				end
				clickables.forth
			end
		end

	show_game_closeup (a_closeup: CLOSEUP) is
			-- Show game closeup
		do
			create closeup.make_from_game_closeup (a_closeup)
			closeup.set_event_action_option_click (agent handle_option_click_event)
			closeup_start := gui_constants.ticks
		end

	load_close_closeup (pdata: ANY) is
			-- load a game into the engine (via serializer) and close closeup
		local
			path: STRING
			transition: GUI_TRANSITION_EXPLODE
		do
			path ?= pdata
			gui.serializer.load_game (path)
			game_skin.main_menu.enable_all

			prepare_scene_transition
			next_scene := create {GUI_GAME_SCENE}.make_from_game_scene (gui.engine.get_current_scene)
			next_scene.initialize_scene

			create transition
			transition.do_transition (Current, next_scene, screen)
		end

	save_close_closeup (pdata: ANY) is
			-- save a game from the engine (via serializer) and close closeup
		local
			path: STRING
			count: INTEGER
		do
			path ?= pdata
			if path.is_integer then
				count := path.to_integer
				-- CHANGES -> save
				-- path := "./saved/Saved Game " + count.out + ".save"
				path := "saved/Saved Game " + count.out + ".save"
			end
			gui.serializer.save_game (path)
			game_skin.main_menu.enable_all
			prepare_for_scene_view
		end

	script_close_closeup (pdata: ANY) is
			-- load a script into the engine (via serializer) and close closeup
		local
			path: STRING
			transition: GUI_TRANSITION_EXPLODE
			--transition: GUI_TRANSITION_DISSOLVE
		do
			path ?= pdata
			gui.parser.set_filename (path)
			gui.parser.startparsing
			if not gui.parser.error_happened then
				gui.engine.set_game (gui.parser.get_game)
				game_skin.main_menu.enable_all

				prepare_scene_transition
				next_scene := create {GUI_GAME_SCENE}.make_from_game_scene (gui.engine.get_current_scene)
				next_scene.initialize_scene

				if not unrecoverable_error then
					create transition
					transition.do_transition (Current, next_scene, screen)
				end
			else
				gui.application.throw_unrecoverable_exception ("Parser error: " + gui.parser.error_string)
			end
		end

	current_game_scene: SCENE
	current_game_closeup: CLOSEUP

end -- class GUI_GAME_SCENE
