indexing

	description: "[

		Represents a transformation and is capable to customize other render parameters.
		Translation is ALWAYS relative to the parent node.
		Orientation can be relative or not (use feature set_inherit_orientation).
		Scale can be relative or not (use feature set_inherit_scale).
		First we apply scaling, second orientation and then translation.

	]"
	date: "$Date$"
	revision: "$Revision$"

deferred class EM3D_NODE

inherit
	ANY
	redefine
		copy
	end

	EM3D_RENDER_PARAMETERS

	EM_NAME_SUPPORT[STRING]
	undefine
		copy, is_equal
	end

	EM3D_POSITIONABLE
	redefine
		set_position, translate
	end
	EM3D_SCALABLE
	redefine
		set_scale, resize
	end
	EM3D_ROTATABLE
	redefine
		set_orientation, rotate
	end


feature{EM3D_NODE, EM3D_SCENE_MANAGER}

	make_by_parent ( a_parent: like Current ) is
			-- Create a new node with a given parent
		do
			create {DS_LINKED_LIST[like Current]} children.make_default
			if a_parent /= Void then
				a_parent.add_child (Current)
			end
			needs_update := True
			scale := scale.one
			transformation.set_unit
		ensure
			parent_set: parent = a_parent
			needs_update_set: needs_update
			children_created: children /= Void
			scale_proper_set: scale = scale.one
		end

feature {NONE} -- Internal
	internal_world_transformation: EM_MATRIX44

feature -- Access world transformation

	world_transformation: like internal_world_transformation is
			-- Transformation in world coordinate system.
		do
			update_from_parent
			result := internal_world_transformation
		end
	world_position: EM_VECTOR3D is
			-- The position of the object in the EM3D_NODE in the world
		do
			result := world_transformation.row( 4 )
		end
	world_orientation: EM_QUATERNION is
			-- The orientation in the world
		do
			result := world_transformation
		end



feature -- Access properties

	parent: like Current
			-- The parent object, might be void.

	children: DS_LIST[ like Current ]
		-- The children of this node.


feature -- Element change

	add_child( a_child: like Current ) is
			-- Add a child
		require
			not_already_in_a_scene: not a_child.has_parent
		do
			if a_child.parent /= void then
				a_child.parent.remove_child( a_child )
			end
			a_child.enable_needs_update
			children.put_last( a_child )
			a_child.set_parent (Current)
		ensure
			has_child: has_child (a_child)
		end

	new_child: like Current is
			-- Create a new child of the current node
		deferred
		ensure
			parent_set: Result.parent = Current
			is_a_child: has_child (Result)
		end

	new_child_by_name (a_name: STRING): like Current is
			-- Create a new child of the current node by name
		require
			a_name_not_void: a_name /= Void
		do
			Result := new_child
			Result.set_name (a_name)
		ensure
			parent_set: Result.parent = Current
			is_a_child: has_child (Result)
			name_set: Result.name.is_equal(a_name)
		end
	remove_child( a_child: like current ) is
			-- Remove a child node
		require
			has_child: has_child( a_child )
		do
			children.delete ( a_child )
			a_child.set_parent ( void )
		end
	remove_from_parent is
			-- Remove the child from its parent
		require
			has_parent: has_parent
		do
			if has_parent then
				parent.remove_child ( Current )
			end
		end


feature -- Position control

	set_position (a_position: EM_VECTOR3D) is
				-- Set `position' to `a_position'.
		do
			Precursor( a_position )
			enable_needs_update
		end

	translate ( a_direction: EM_VECTOR3D ) is
			-- Translate the object
		do
			Precursor( a_direction )
			enable_needs_update
		end

feature -- orientation control

	set_orientation (a_orientation: EM_QUATERNION) is
				-- Set `orientation' to `a_orientation'.
		do
			Precursor( a_orientation )
			enable_needs_update
		end

	rotate(a_orientation: EM_QUATERNION) is
				-- Rotate the object with a quaternion.
		do
			Precursor( a_orientation )
			enable_needs_update
		end


feature -- Scale control

	set_scale (a_scale: EM_VECTOR3D) is
				-- Set `scale' to `a_scale'.
		do
			Precursor( a_scale )
			enable_needs_update
		end

	resize( a_scale: EM_VECTOR3D ) is
			-- Resize the object by `a_scale'
		do
			Precursor( a_scale )
			enable_needs_update
		end

feature {EM3D_NODE} -- Element change

	set_parent (a_parent: like Current) is
			-- Sets parent to `a_parent'.
		do
			parent := a_parent
			enable_needs_update
		ensure
			parent_set: parent = a_parent
		end


feature -- Status
	is_normalizing_normals: BOOLEAN
			-- Should the EM3D system automatically normalize the normals?
			-- Use this feature if your normals are not normalized.
			-- Without normalized normals some computation, like lighting, yield incorrect results.

	has_any_children: BOOLEAN is
			-- An object is not dead if it has children
		do
			Result := not children.is_empty
		end

	has_child (a_child: like Current): BOOLEAN is
			-- Is `a_child' a child in our list?
		do
			Result := children.has (a_child)
		end

	has_parent: BOOLEAN is
			-- Do we have a parent?
		do
			Result := parent /= Void
		end


	is_inheriting_orientation_from_parent: BOOLEAN is
			-- Is orientation inherited from parent node?
		deferred
		end

	is_inheriting_scale_from_parent: BOOLEAN is
			-- Is scale inherited from parent node?
		deferred
		end


feature {EM3D_NODE} -- Status setting

	enable_needs_update is
			-- Sets `is_needing_update' to true.
		local
			c: DS_LIST_CURSOR[ like current ]
		do
			if not needs_update then
				from c := children.new_cursor; c.start until c.after loop
					c.item.enable_needs_update
					c.forth
				end
			end
			needs_update := True
		ensure
			is_enabled: needs_update
		end

feature {EM3D_NODE} -- Status

	needs_update: BOOLEAN
			-- Does the current node need an update?

	update_from_parent is
			-- Update the current world transformation if needed
		do
			if needs_update then
				if parent/=void then
					internal_world_transformation := parent.world_transformation * transformation
				else
					internal_world_transformation := transformation
				end
				internal_world_transformation.set_row ( internal_world_transformation.row (1).scaled ( scale.x ), 1)
				internal_world_transformation.set_row ( internal_world_transformation.row (2).scaled ( scale.y ), 2)
				internal_world_transformation.set_row ( internal_world_transformation.row (3).scaled ( scale.z ), 3)
				needs_update := false
			end
		end


end -- class EM3D_NODE
