/*
trie.c

Copyright (C) 2006  Philipp Kraehenbuehl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "trie.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

TrieNode * newTrieNode( char* key, TRIE_TYPE element ){
	TrieNode *n = (TrieNode*) malloc( sizeof(TrieNode) );
	n->key = strdup( key );
	n->element = element;
	n->next = NULL;
	n->down = NULL;
	return n;
}

void freeTrieNode( TrieNode * nd ){
	free( nd->key );
	nd->key = NULL;
	free( nd );
}

void freeTrieNodeRecurive( TrieNode * nd ){
	if (nd) {
		freeTrieNodeRecurive( nd->next );
		freeTrieNodeRecurive( nd->down );
		free( nd->key );
		nd->key = NULL;
		free( nd );
	}
}


Trie * newTrie (void){
	Trie * n = (Trie*)malloc( sizeof(Trie) );
	n->root = NULL;
	return n;
}

void freeTrie ( Trie * t ){
	if (t->root)
		freeTrieNodeRecurive( t->root );
	free( t );
}

// Insert funtions

TrieNode * TrieNodeSplitNode( TrieNode ** nd, char * k1, char * k2, TRIE_TYPE element ){
	TrieNode * t1 = NULL;
	TrieNode * t2 = NULL;
	TrieNode * t0 = NULL;
	TrieNode * tt;
	char t; char * tmp;
	
	if (*k1 && *k2){
		t = *k1; *k1 = 0;
		t1 = *nd;
		t0 = newTrieNode( t1->key, (TRIE_TYPE)0 );
		*k1 = t; t0->next=t1->next;t1->next=NULL;
		t2 = newTrieNode( k2, element );
		tmp = t1->key; t1->key = strdup( k1 );
		*nd = t0;
		
		tt = t2;
		if (*k1 > *k2){
			tt = t2; t2 = t1; t1 = tt;
		}
		free(tmp);
		t1->next = t2;
		t0->down = t1;
		return tt;
	} else if (*k1){
		t = *k1; *k1 = 0;
		t0 = newTrieNode( (*nd)->key, element );
		t0->down = (*nd); t0->next = (*nd)->next; (*nd)->next = NULL;
		*k1 = t;
		tmp = (*nd)->key; (*nd)->key = strdup( k1 ); free( tmp );
		(*nd) = t0;
		return t0;
	} else
		perror( "Split missused\n" );
	assert( 0 ); 
}

TrieNode * TrieNodeInsert( TrieNode ** nd, char* key, TRIE_TYPE element ){
	char * k1 = (*nd)->key;
	char * k2 = key;
	TrieNode * t1 = NULL;
	int i;
	
	while (*k2){
		k1 = (*nd)->key;
		
		for (i=0; *k2==*k1 && *k1!=0; i++){
			k1++; k2++;
		}
		
		if (i==0){
			if (*k1 > *k2){
				t1 = newTrieNode( k2, element );
				t1->next = *nd;
				*nd = t1;
				return t1;
			}else if (!(*nd)->next)
				return (*nd)->next = newTrieNode( k2, element );
			else
				nd = & (*nd)->next;
		}
		//Both are 0 => k1==k2
		else if (!*k1 && !*k2){
			(*nd)->element = element;
			return *nd;
		}
		else if (!*k1){
			if ((*nd)->down)
				nd = & (*nd)->down;
			else
				return (*nd)->down = newTrieNode( k2, element );
		}
		else
			return TrieNodeSplitNode( nd, k1, k2, element );
	}
	return NULL;
}


TrieNode * TrieInsert( Trie * t, char* key, TRIE_TYPE element ){
	if ( !t->root )
		return t->root = newTrieNode( key, element );
	else
		return TrieNodeInsert( & t->root, key, element );
}

void TrieNodeForeach( TrieNode * t, TrieCallback c ){
	if (!t) return;
	if( t->element )
		c( t->element );
	if (t->next)
		TrieNodeForeach( t->next, c );
	if (t->down)
		TrieNodeForeach( t->down, c );
}
void TrieForeach( Trie * t, TrieCallback c ){
	if (t)
		TrieNodeForeach( t->root, c );
}
// Search function
TrieNode ** TrieNodeSearch( TrieNode **t, char* key ){
	char * k1 = key;
	char * k2 = NULL;
	
	if (!*t)
		return NULL;
	
	while ( *k1 ){
		k2 = (*t)->key;
		while ( *k1 > *k2 ){
			t = & (*t)->next;
			if (!*t) return NULL;
			k2 = (*t)->key;
		}
		
		while (*k1!=0 && *k2!=0){
			if (*k1 != *k2) return NULL;
			k1++; k2++;
		}
		
		if ( *k1 ){
			t = & (*t)->down;
			if (!*t) return NULL;
		}
		else if ( *k2 ) return NULL;
		else return t;
	}
	return NULL;
}


TRIE_TYPE TrieSearch( Trie * t, char* key ){
	TrieNode ** nd = TrieNodeSearch( & t->root, key );
	if (nd && *nd)
		return (*nd)->element;
	else
		return (TRIE_TYPE)0;
}

// Remove functions
void TrieNodeRemove( TrieNode **nd ){
	TrieNode * t;
	int n, m;
	if ( (*nd)->down ){
		// If there is more than one child do not delete (This does NOT worsen the memory usage).
		// Or only by a factor of 2
		if ( (*nd)->down->next ){
			(*nd)->element = (TRIE_TYPE)0;
			t = *nd;
		}
		else{
			n = strlen((*nd)->key);
			t = (*nd)->down;
			m = strlen(t->key);
			t->key = realloc( t->key, (m + n + 1) * sizeof(char) );
			memmove( t->key+n, t->key, m+1 );
			memcpy( t->key, (*nd)->key, n );
			
			t->next = (*nd)->next;
			t = (*nd)->down;
			freeTrieNode( (*nd) );
		}
	}
	else{
		t = (*nd)->next;
		freeTrieNode( (*nd) );
	}
	*nd = t;
}

int TrieNodeRemoveMerge( TrieNode ** t, char * key ){
	char * k1 = key;
	char * k2 = NULL;
	
	if (!*t)
		return 0;
	
	while ( *k1 ){
		k2 = (*t)->key;
		while ( *k1 > *k2 ){
			t = & (*t)->next;
			if (!*t) return 0;
			k2 = (*t)->key;
		}
		
		while (*k1!=0 && *k2!=0){
			if (*k1 != *k2) return 0;
			k1++; k2++;
		}
		
		if ( *k1 ){
			if (!(*t)->down) return 0;
			// Merge
			if ( !(*t)->down->element && !((*t)->down->next) ){
				// Merge the stirng
				int n = strlen((*t)->key);
				TrieNode * tt = (*t)->down;
				int m = strlen(tt->key);
				tt->key = realloc( tt->key, (m + n + 1) * sizeof(char) );
				memmove( tt->key+n, tt->key, m+1 );
				memcpy( tt->key, (*t)->key, n );
				
				tt->next = (*t)->next;
				tt = (*t)->down;
				freeTrieNode( (*t) );
				*t = tt;
			}
		}
		else if ( *k2 ) return 0;
		else return 1;
	}
	return 0;
}


void TrieRemove( Trie *t, char* key ){
	TrieNode ** nd = TrieNodeSearch( & t->root, key );
	if (nd && *nd){
		TrieNodeRemove( nd );
		TrieNodeRemoveMerge( nd, key );
	}
}
