indexing

	description: "[

		Event object that is secured with a two phase commit system.
		
		You may set a `success_agent' that is called if all remote peers
		have sent an acknowledgment. In this case you can assume that all
		remote peers that belong to the sending group have taken the appropriate
		actions.
		
		You may also set a `timeout_agent' that is called if at least one
		acknowledgement for a remote peer is missing after some period of time.
		You'll get a list of clients that have acknowledements missing as argument.
		In this case you can not be sure if only the acknowledgement has been
		lost, or the remote peer failed to handle the action at all. You'll need
		to take an appropriate action, like dropping the non answering client to
		make the system consistent again.
		
		If an acknowledgement is missing for a connection after `resend_time', the
		event will be sent again for at most `resends_left' times.
		If only the acknowledgement was lost in the previous attempt,
		then only the acknowledgement will be sent again, but the event will
		NOT be fired again on the client side.
		
		The 2PC system relies on the object ID. If you define your own 2PC events
		you need to set a unique ID for every instance of an event (use a dedicated
		EM_NET_OJBECT_ID_MANAGER, like the one supplied in EM_NET_BASE, for that purpose)
		and serialize/unserialize it manually.

	]"
	date: "$Date: 2006-02-08 13:35:06 +0100 (Wed, 08 Feb 2006) $"
	revision: "$Revision: 1695 $"

deferred class	EM_NET_2PC_EVENT_OBJECT

inherit

	EM_NET_EVENT_OBJECT
		redefine
			make_from_stream,
			publish
		end

feature {NONE} -- Initialization

	make_from_stream (an_unserializer: EM_NET_UNSERIALIZER) is
			-- Create a 2PC event from a stream and automatically acknowledge it.
		do
			Precursor (an_unserializer)
			group.send_ack (id)
		end

feature -- Serialization

	serialize (a_serializer: EM_NET_SERIALIZER) is
			-- Serialize to the given stream using `a_serializer'.
		require else
			a_serializer_not_void: a_serializer /= Void
			id_set: id > 0
		deferred
		end

	unserialize (an_unserializer: EM_NET_UNSERIALIZER) is
			-- Unserialize from a given stream using `an_unserializer'.
		deferred
		ensure then
			id_set: id > 0
		end

feature -- Event handling

	publish is
			-- Publish an event to the connections that belong to `group'.
			-- Setup 2PC variables in EM_NET_BASE.
		local
			event_sockets_pair: EM_PAIR[EM_NET_2PC_EVENT_OBJECT,DS_LINKED_LIST[EM_NET_CONNECTION]]
			connection_list: DS_LINKED_LIST[EM_NET_CONNECTION]
		do
			create connection_list.make_from_linear (group.connections)
			create event_sockets_pair.make (Current, connection_list)
			group.base.pending_2pc_acks.force (event_sockets_pair, id)
			group.net_event_container_object.publish (Current)
			group.base.enable_2pc_timed_resend (Current)
		end

feature -- Element change

	set_success_agent (a_success_agent: PROCEDURE[ANY,TUPLE[]]) is
			-- Set `success_agent' to `a_success_agent'.
			-- `a_success_agent' is called if 2PC was successful.
		do
			success_agent := a_success_agent
		end

	set_timeout_agent (a_timeout_agent: PROCEDURE[ANY,TUPLE[DS_LINKED_LIST[EM_NET_CONNECTION]]]) is
			-- Set `timeout_agent' to `a_timeout_agent'.
			-- `a_timeout_agent' is called if 2PC was not successful after
			-- some period of time.
			-- The handler will get a DS_LINKED_LIST[EM_NET_CONNECTION] as argument that
			-- contains a list of connections that did not respond in time.
		do
			timeout_agent := a_timeout_agent
		end

	set_resend_time (a_time: INTEGER) is
			-- Set `resend_time' to `a_time'.
			-- `a_time' is in milliseconds.
		require
			a_time_not_negative: a_time >= 0
		do
			resend_time := a_time
		end

	set_resends_left (a_number: INTEGER) is
			-- Set `resends_left' to `a_number'.
		require
			a_number_not_negative: a_number >= 0
		do
			resends_left := a_number
		end

feature -- Access

	success_agent: PROCEDURE[ANY,TUPLE[]]
			-- Agent that is called after a successful two phase commit

	timeout_agent: PROCEDURE[ANY,TUPLE[DS_LINKED_LIST[EM_NET_CONNECTION]]]
			-- Agent that is called after a timeout occurred in a two phase commit

	resend_time: INTEGER
			-- Time in ms after the event is resent if no ACK was received.

	resends_left: INTEGER
			-- Number of event resends left
			-- An event will be resent if no ACK was received after `resend_time' until
			-- `resends_left' reaches 0.
			-- This value will decrease automatically.

end
