indexing
	description: "[
	
		A resizeable axis box that contains a graphical representation
		of a coordinate axis of a standard scientific coordinate system
		(right-handed, z-axis towards the viewer, just like OpenGL).
		Y-axis and Z-axis are inverted by default because of the above.
		Axes are normally used in conjunction with plots or renderer-boxes,
		which use the same standard coordinate system.
		
		Axes are highly customizable. Size of box and axis position inside
		box are determined by placement of labels.

		Labels and blips are created automatically when updating.
		Using a custom labeler, it is possible to control the text of the
		labels.

	]"
	author: ""
	date: "$Date: 2007-02-19 07:06:31 +0100 (Mon, 19 Feb 2007) $"
	revision: "$Revision: 2538 $"

class
	EM_VIZ_LINEAR_AXIS_BOX

inherit
	EM_VIZ_AXIS_BOX

create

	make,
	make_x,
	make_y,
	make_z

feature -- Initialization

	make (a_length: like axis_length; an_index: like axis_index; inverted: BOOLEAN; an_interval: like interval) is
			-- Make axis of `a_length' that covers values in `an_interval'
		do
			make_axis (a_length, an_index, inverted)
			set_labeler (Void)
			set_interval (an_interval)
			set_label_start (an_interval.min)
			set_label_spacing (an_interval.size)
			set_blip_start (an_interval.min)
			set_blip_spacing (an_interval.size)
		end

	make_x (a_length: like axis_length; an_interval: like interval) is
			-- Make x-axis of `a_length' representing the values in `an_interval'
		do
			make (a_length, X_axis, False, an_interval)

			set_label_alignment ([Align_right, Align_center])
			set_label_rotation (-90.0)
			put_head_arrow
		end

	make_y (a_length: like axis_length; an_interval: like interval) is
			-- Make y-axis of `a_length' representing the values in `an_interval'
			-- Because Viz uses a coordinate system with the Y-axis pointing downwards,
			-- we invert the axis by default
		do
			make (a_length, Y_axis, True, an_interval)

			set_label_alignment ([Align_right, Align_center])
			put_head_arrow
		end

	make_z (a_length: like axis_length; an_interval: like interval) is
			-- Make z-axis of `a_length' representing the values in `an_interval'
			-- Because Viz uses a coordinate system with the Z-axis pointing away from
			-- the viewer, we invert the axis by default
		do
			make (a_length, Z_axis, True, an_interval)

			set_label_alignment ([Align_left, Align_center])
			set_label_rotation (45.0)
			put_head_arrow
		end

feature -- Access

	interval: EM_INTERVAL
			-- Range of values that axis represents

	blip_start: DOUBLE
			-- Blip start

	blip_spacing: DOUBLE
			-- Blip spacing

	label_start: DOUBLE
			-- Label start

	label_spacing: DOUBLE
			-- Label spacing

	labeler: EM_VIZ_LABELER
			-- Labeler that produces text for labels

feature -- Element change

	set_labeler (a_labeler: like labeler) is
			-- Set a labeler
		do
			labeler := a_labeler
			expire
		ensure
			set: labeler = a_labeler
			expired: needs_update
		end

	set_interval (an_interval: like interval) is
			-- Set value-range for axis
		do
			interval := an_interval
			expire
		ensure
			set: interval = an_interval
			expired: needs_update
		end

	set_blip_start (a_pos: like blip_start) is
			-- Set blip start
		do
			blip_start := a_pos
			expire
		ensure
			set: blip_start = a_pos
			expired: needs_update
		end

	set_blip_spacing (a_size: like blip_spacing) is
			-- Set blip spacing
		require
			positive_size: a_size > 0.0
		do
			blip_spacing := a_size
			expire
		ensure
			set: blip_spacing = a_size
			expired: needs_update
		end

	set_label_start (a_pos: like label_start) is
			-- Set label start
		do
			label_start := a_pos
			expire
		ensure
			set: label_start = a_pos
			expired: needs_update
		end

	set_label_spacing (a_size: like label_spacing) is
			-- Set label spacing
		require
			positive_size: a_size > 0.0
		do
			label_spacing := a_size
			expire
		ensure
			set: label_spacing = a_size
			expired: needs_update
		end

feature -- Manufacturing

	create_simple_labeler (a_prefix, a_postfix: STRING) is
			-- Create a simple labeler for this axis
		do
			labeler := create {EM_VIZ_SIMPLE_LABELER}.make (a_prefix, a_postfix)
		ensure
			labeler_exists: labeler /= Void
		end

feature {NONE} -- Implementation

	value_to_fraction (v: DOUBLE): DOUBLE is
			-- Convert from value in range to fraction of axis
		do
			Result := (v - interval.min) / interval.size
		end

	update_labels is
			-- Update labels
		local
			i: INTEGER
			v: DOUBLE
		do
			if labeler /= Void then
				from
					i := 1
					v := label_start
				until
					v > interval.max
				loop
					Current.put_label (value_to_fraction (v), labeler.label (i, v))
					i := i + 1
					v := v + label_spacing
				end
			end
		end

	update_blips is
			-- Update blips
		local
			v: DOUBLE
		do
			from
				v := blip_start
			until
				v > interval.max
			loop
				put_blip (value_to_fraction (v))
				v := v + blip_spacing
			end
		end

end
