indexing
	description: "[

		A resizeable box that contains a pie-chart which can be rotated.
		Slices need to be added to the chart before a pie will become visible.
		Slices can have an explicit color or they will be assigned a color
		automatically by the chart based on the range of hues given.

	]"
	author: ""
	date: "$Date: 2007-02-18 01:09:05 +0100 (Sun, 18 Feb 2007) $"
	revision: "$Revision: 2532 $"

class
	EM_VIZ_PIE_CHART

inherit
	EM_VIZ_BOX
		redefine
			update,
			vz_cache
		end

	EM_VIZ_RESIZEABLE

	EM_VIZ_ROTATEABLE

create

	make

feature -- Initialization

	make (a_size: like size) is
			-- Make empty pie-chart of `a_size'
		do
			make_box
			make_resizeable (a_size)
			make_rotateable

			-- Optimal!
			set_label_offset (optimal_label_offset)
			set_pie_radius (optimal_pie_radius)
			set_pie_depth (optimal_pie_depth)
			-- Full color spectrum
			set_hue_range ([0.0, 360.0])

			create slices.make (1)
			create fractions.make (1)
			create labels.make (1)
			create coordinate_frame.make_unit
		ensure
			slices_exist: slices /= Void
			fractions_exist: fractions /= Void
			labels_exist: labels /= Void
			coordinate_frame_exist: coordinate_frame /= Void
		end

feature -- Access

	hue_range: EM_INTERVAL
			-- Hue-range is used to color slices without explicit color

	label_offset: DOUBLE
			-- Label offset from pie

	pie_radius: DOUBLE
			-- Radius of pie

	pie_depth: DOUBLE
			-- Depth of pie

	optimal_label_offset: DOUBLE is
			-- Optimal label offset given the current measurements
		do
			Result := Viz_options.label_font_format.line_skip
		ensure
			positive: 0.0 < Result
		end

	optimal_pie_radius: DOUBLE is
			-- Optimal pie radius given the current measurements
		do
			Result := width.min (height) / 2.0 - label_offset - Viz_options.label_font_format.line_skip
		ensure
			positive: 0.0 < Result
		end

	optimal_pie_depth: DOUBLE is
			-- Optimal pie depth given the current measurements
		do
			Result := 0.2 * optimal_pie_radius
		ensure
			positive: 0.0 < Result
		end

feature -- Element change

	set_label_offset (a_offset: like label_offset) is
			-- Set label offset
		require
			offset_positive: 0.0 < a_offset
		do
			label_offset := a_offset
			expire
		ensure
			set: label_offset = a_offset
			expired: needs_update
		end

	set_pie_depth (a_depth: like pie_depth) is
			-- Set pie depth
		do
			pie_depth := a_depth
			expire
		ensure
			set: pie_depth = a_depth
			expired: needs_update
		end

	set_pie_radius (a_radius: like pie_radius) is
			-- Set pie radius
		do
			pie_radius := a_radius
			expire
		ensure
			set: pie_radius = a_radius
			expired: needs_update
		end

	set_hue_range (a_range: EM_INTERVAL) is
			-- Set hue range
		do
			hue_range := a_range
			expire
		ensure
			set: hue_range = a_range
			expired: needs_update
		end

	set_slices (some_slices: DS_LINEAR [EM_VIZ_PIE_SLICE]) is
			-- Set slices of pie to `some_slices'
		do
			slices.wipe_out
			slices.append_last (some_slices)
			expire
		ensure
			expired: needs_update
		end

feature {NONE} -- Rendering

	vz_cache is
			-- Cache labels
		do
			from
				labels.start
			until
				labels.off
			loop
				labels.item_for_iteration.cache
				labels.forth
			end
		end

	vz_render (viewing_direction: EM_VECTOR3D) is
			-- Render pie slices and labels
		local
			fc: DS_LINEAR_CURSOR [DOUBLE]
			fsum: DOUBLE
		do
			vz_push_frame
			vz_translate (bounds.center)
			vz_mult_frame (coordinate_frame)

			vz_push_attributes (Vz_depth_buffer_bit | Vz_polygon_bit)
			vz_set_depth_test (True, True)
			vz_set_culling (True, Vz_back)
			from
				fsum := 0.0
				fc := fractions.new_cursor
				fc.start
				slices.start
			until
				fc.off or slices.off
			loop
				render_pie_slice (
					default_step, fsum, fsum + fc.item,
					slices.item_for_iteration
				)
				fsum := fsum + fc.item
				fc.forth
				slices.forth
			end

			vz_pop_attributes
			vz_pop_frame

			vz_push_frame
			vz_translate (bounds.center)
			from
				fsum := 0.0
				fc.start
				slices.start
				labels.start
			until
				fc.off or slices.off or labels.off
			loop
				render_pie_label (
					fsum, fsum + fc.item,
					labels.item_for_iteration
				)
				fsum := fsum + fc.item
				fc.forth
				slices.forth
				labels.forth
			end
			vz_pop_frame
		end

	render_pie_label (frac1, fract2: DOUBLE; label: EM3D_STRING) is
			-- Render pie labels
		local
			p1, p2, v: EM_VECTOR3D
			angle: DOUBLE
			r: DOUBLE
		do
			angle := angle_from_fraction (0.5 * (frac1 + fract2))
			r := pie_radius

			p1 := coordinate_frame.map_point (label_vertex (angle, r + label_offset))
			p2 := coordinate_frame.map_point (label_vertex (angle, r))
			v := p1 - coordinate_frame.origin

			render_linked_label (p1, p2, label)
		end

	render_pie_slice (step, fract1, fract2: DOUBLE; slice: EM_VIZ_PIE_SLICE) is
			-- Renders a slice of pie of unit height and radius
			-- render_pie_slice (1.0/facets, 0, 1) draws
			-- a faceted cylinder of unit radius with axis from (0,0,1)
			-- to (0,0,0), you get the idea
		local
			angle, fractmid: DOUBLE
			angle_step, angle_start, angle_end: DOUBLE
			r, d: DOUBLE
		do
			fractmid := 0.5 * (fract1 + fract2)
			angle_start := angle_from_fraction (fract1)
			angle_step := angle_from_fraction (step)
			angle_end := angle_from_fraction (fract2)
			angle := angle_start
			r := pie_radius
			d := pie_depth

			-- Sides
			vz_begin_quad_strip
				if slice.color /= Void then
					vz_color (slice.color)
				else
					pie_color (fractmid)
				end
				from
					angle := angle_start
				until
					angle >= angle_end
				loop
					pie_normal (angle)
					pie_vertex (angle, r, 0.0)
					pie_vertex (angle, r, d)
					angle := angle + angle_step
				end
				pie_normal (angle)
				pie_vertex (angle_end, r, 0.0)
				pie_vertex (angle_end, r, d)
			vz_end

			-- Top
			vz_begin_triangle_fan
				if slice.color /= Void then
					vz_color (slice.color)
				else
					pie_color (fractmid)
				end
				vz_normal3d (0, 0, -1)
				vz_vertex3d (0, 0, 0)
				from
					angle := angle_start
				until
					angle >= angle_end
				loop
					pie_vertex (angle, r, 0)
					angle := angle + angle_step
				end
				pie_vertex (angle_end, r, 0)
			vz_end

			-- Bottom
			vz_begin_triangle_fan
				if slice.color /= Void then
					vz_color (slice.color)
				else
					pie_color (fractmid)
				end
				vz_normal3d (0, 0, 1)
				vz_vertex3d (0, 0, d)
				from
					pie_vertex (angle_end, r, d)
					angle := angle - angle_step
				until
					angle <= angle_start
				loop
					pie_vertex (angle, r, d)
					angle := angle - angle_step
				end
				pie_vertex (angle_start, r, d)
			vz_end
		end

feature -- Update mechanism

	update is
			-- Update everything
		do
			update_frame
			update_fractions
			update_labels
			Precursor
		end

feature {NONE} -- Implementation

	update_frame is
			-- Update coordinate frame from rotation
		do
			coordinate_frame.make_unit
			coordinate_frame.rotate (rotation_angle, rotation_axis)
		end

	update_labels is
			-- Update labels
		local
			l: EM3D_STRING
		do
			labels.resize (slices.count)
			labels.wipe_out

			from
				slices.start
			until
				slices.off
			loop
				create l.make_with_format (
					slices.item_for_iteration.label_text,
					Viz_options.label_font_format
				)
				labels.put_last (l)
				slices.forth
			end
		end

	update_fractions is
			-- Update fractions (each slice only takes a fraction of the pie)
		local
			total: DOUBLE
		do
			fractions.resize (slices.count)
			fractions.wipe_out

			from
				slices.start
				total := 0.0
			until
				slices.off
			loop
				total := total + slices.item_for_iteration.value
				slices.forth
			end

			if total = 0.0 then
				total := 1.0
			end

			from
				slices.start
			until
				slices.off
			loop
				fractions.force_last (slices.item_for_iteration.value / total)
				slices.forth
			end
		end

	frozen angle_from_fraction (frac: DOUBLE): DOUBLE is
			-- Angle from fraction
		do
			Result := 2.0 * Pi * frac
		end

	frozen degrees_from_fraction (frac: DOUBLE): DOUBLE is
			-- Degrees from fraction
		do
			Result := 360.0 * frac
		end

	pie_vertex (angle, r, z: DOUBLE) is
			-- Vertex for `angle', radius `r' and depth `z'
		do
			vz_vertex3d (r * sine (angle), - r * cosine (angle), z)
		end

	pie_normal (angle: DOUBLE) is
			-- Normal for `angle'
		do
			vz_normal3d (sine (angle), - cosine (angle), 0.0)
		end

	label_vertex (angle, r: DOUBLE): EM_VECTOR3D is
			-- Vertex for `angle' and radius `r'
		do
			Result.set (r * sine (angle), - r * cosine (angle), -0.01)
		end

	pie_color (frac: DOUBLE) is
			-- Implicit color for slice staring at `frac'
		local
			h: DOUBLE
			r, g, b: DOUBLE
		do
			h := hue_range.min + hue_range.size * frac
			h := h * 6.0 / 360.0

			if h < 0.0 then h := h + 6.0 end
			if h > 6.0 then h := h - 6.0 end

			if h < 1.0 then
				r := (1.0    )
				g := (h      )
			elseif h < 2.0 then
				r := (2.0 - h)
				g := (1.0    )
			elseif h < 3.0 then
				g := (1.0    )
				b := (h - 2.0)
			elseif h < 4.0 then
				g := (4.0 - h)
				b := (1.0    )
			elseif h < 5.0 then
				b := (1.0    )
				r := (h - 4.0)
			else
				b := (6.0 - h)
				r := (1.0    )
			end
			vz_color3d (r, g, b)
		end

	slices: DS_ARRAYED_LIST [EM_VIZ_PIE_SLICE]
			-- All slices that make up pie

	fractions: DS_ARRAYED_LIST [DOUBLE]
			-- Fractions of pie (how much does each slice contribute?)

	labels: DS_ARRAYED_LIST [EM3D_STRING]
			-- Labels for every slice

	default_step: DOUBLE is 0.025
			-- Default fraction step for rendering

	coordinate_frame: EM_VIZ_COORDINATE_FRAME
			-- Local coordinate frame of pie

invariant

	slices_exist: slices /= Void
	fractions_exist: fractions /= Void
	labels_exist: labels /= Void
	coordinate_frame_exist: coordinate_frame /= Void

end
