indexing
	description: "[
	
		A box that has a local coordinate frame and and parameters that control rendering.
		By default, the rendering environment is set up to have the same right-handed
		coordinate-system as OpenGL. Again, this can be overriden by a more advanced
		user of the library.
		Clipping can be controlled for each axis individually.

	]"
	author: ""
	date: "$Date: 2007-02-19 11:34:20 +0100 (Mon, 19 Feb 2007) $"
	revision: "$Revision: 2545 $"

deferred class
	EM_VIZ_RENDERER_BOX

inherit
	EM_VIZ_BOX
		redefine
			update, vz_setup, vz_revert
		end

	EM_VIZ_RESIZEABLE

feature -- Initialize

	make_renderer_box (a_size: like size; an_interval: like interval) is
			-- Make new renderer box with standard right-handed coordinate system like OpenGL
			-- Depth-testing and -writing is enabled by default
			-- Clipping is enabled for x- and y-coordinates, but disabled for z-coordinates
			-- Culling is disabled
		do
			make_box
			make_resizeable (a_size)
			set_clipping_enabled (True, True, True)
			set_clipping_offset (1.0)
			set_culling (False, Vz_front_and_back)
			set_depth_test (True, True)
			-- Set up right-handed coordinate system (same as standard OpenGL)
			set_interval (an_interval)
			set_axes_inverted (False, True, True)

			create coordinate_frame.make_unit
		end

feature -- Access

	depth_test: BOOLEAN
			-- Should depth testing be enabled

	depth_write: BOOLEAN
			-- Should depth values be written to the depth-buffer?
			-- (independent of `depth_test')

	culling_enabled: BOOLEAN
			-- Should culling be enabled?

	cull_face: like Vz_front_and_back
			-- If culling is enabled, which faces should be culled?

	x_clipping_enabled: BOOLEAN
			-- Is clipping enabled for x-coordinates?

	y_clipping_enabled: BOOLEAN
			-- Is clipping enabled for x-coordinates?

	z_clipping_enabled: BOOLEAN
			-- Is clipping enabled for x-coordinates?

	clipping_offset: DOUBLE
			-- The closer to zero this value is, the tighter the clipping planes will be to the
			-- Box

	interval: EM_INTERVAL_3D
			-- Range of values inside box

 	x_axis_inverted: BOOLEAN
			-- Should x-axis be inverted with respect to Viz coordinate-frame?

	y_axis_inverted: BOOLEAN
			-- Should y-axis be inverted with respect to Viz coordinate-frame?

	z_axis_inverted: BOOLEAN
			-- Should z-axis be inverted with respect to Viz coordinate-frame?

feature -- Status setting

	set_culling (enable: BOOLEAN; face: like vz_front_and_back) is
			-- Set culling parameters
		do
			culling_enabled := enable
			cull_face := face
			expire
		ensure
			set: culling_enabled = enable and cull_face = face
			expired: needs_update
		end

	set_depth_test (enable: BOOLEAN; write: BOOLEAN) is
			-- Set depth test parameters
		do
			depth_test := enable
			depth_write := write
			expire
		ensure
			set: depth_test = enable and depth_write = write
			expired: needs_update
		end

	set_clipping_enabled (x_enable, y_enable, z_enable: BOOLEAN) is
			-- Selectively enable clipping-planes
		do
			x_clipping_enabled := x_enable
			y_clipping_enabled := y_enable
			z_clipping_enabled := z_enable
			expire
		ensure
			set:
				x_clipping_enabled = x_enable and
				y_clipping_enabled = y_enable and
				z_clipping_enabled = z_enable
			expired: needs_update
		end

	set_clipping_offset (an_offset: like clipping_offset) is
			-- Set clipping offset
		do
			clipping_offset := an_offset
			expire
		ensure
			set: clipping_offset = an_offset
			expired: needs_update
		end

feature -- Element change

	set_axes_inverted (invert_x, invert_y, invert_z: BOOLEAN) is
			-- Set inversion of each axis
		do
			x_axis_inverted := invert_x
			y_axis_inverted := invert_y
			z_axis_inverted := invert_z
			expire
		ensure
			set:
				x_axis_inverted = invert_x
				y_axis_inverted = invert_y
				z_axis_inverted = invert_z
			expired: needs_update
		end

	set_interval (an_interval: like interval) is
			-- Set all intervals at once
		do
			interval := an_interval
			expire
		ensure
			set: interval = an_interval
			expired: needs_update
		end

feature -- Rendering

	vz_setup is
			-- Setup clipping planes and local coordinate frame
		local
			pmin, pmax: EM_VECTOR3D
		do
			Precursor

			pmin := bounds.min
			pmax := bounds.max

			if x_clipping_enabled or y_clipping_enabled or z_clipping_enabled then
				vz_push_attributes (Vz_polygon_bit | Vz_depth_buffer_bit | Vz_transform_bit)
			else
				vz_push_attributes (Vz_polygon_bit | Vz_depth_buffer_bit)
			end

			if x_clipping_enabled then
				vz_enable_clip_plane (Vz_clip_plane0, 1, 0, 0, -pmin.x + clipping_offset)
				vz_enable_clip_plane (Vz_clip_plane1,-1, 0, 0,  pmax.x + clipping_offset)
			end
			if y_clipping_enabled then
				vz_enable_clip_plane (Vz_clip_plane2, 0, 1, 0, -pmin.y + clipping_offset)
				vz_enable_clip_plane (Vz_clip_plane3, 0,-1, 0,  pmax.y + clipping_offset)
			end
			if z_clipping_enabled then
				vz_enable_clip_plane (Vz_clip_plane4, 0, 0, 1, -pmin.z + clipping_offset)
				vz_enable_clip_plane (Vz_clip_plane5, 0, 0,-1,  pmax.z + clipping_offset)
			end

			vz_set_depth_test (depth_test, depth_write)
			vz_set_culling (culling_enabled, cull_face)

			vz_push_frame
			vz_translate (bounds.min)
			vz_mult_frame (coordinate_frame)
		end

	vz_revert is
			-- Revert clipping status and coordinate system
		do
			vz_pop_attributes
			vz_pop_frame

			Precursor
		end

feature -- Update mechanism

	update is
			-- Update internals
		do
			Precursor
			update_frame
		end

feature {NONE} -- Implementation

	coordinate_frame: EM_VIZ_COORDINATE_FRAME
			-- Mapping from points inside box to Viz frame

	update_frame is
			-- Update mapping
		local
			size_interval: like bounds
		do
			size_interval.set ([0.0,0.0,0.0], size)

			coordinate_frame.make_box_mapping_with_inverted_axes (
				size_interval, interval,
				x_axis_inverted,
				y_axis_inverted,
				z_axis_inverted
			)
		end

invariant

	coordinate_frame_exists: coordinate_frame /= Void

end
