indexing
	description: "[
	
		Updateable objects that can render themselves using OpenGL, with support
		for display lists.
		Objects will be automatically updated, cached, and possibly compiled before rendering, but
		only if the object is not up-to-date already. 
		A subclass should always redefine both vz_setup AND vz_revert, if either is redefined.
		This class also provides some standard rendering functions e.g. for bounding-boxes and the like.
		
		See also EM_VIZ_RENDERER

	]"
	author: ""
	date: "$Date: 2007-02-12 06:18:18 +0100 (Mon, 12 Feb 2007) $"
	revision: "$Revision: 2504 $"

deferred class
	EM_VIZ_RENDERABLE

inherit

	EM_ALIGNMENT_CONSTANTS
		export {EM_VIZ_RENDERABLE} all end

	EM_VIZ_RENDERER
		export {EM_VIZ_RENDERABLE} all end

	EM_VIZ_MATH
		export {EM_VIZ_RENDERABLE} all end

	EM_VIZ_UPDATEABLE

feature -- Initialization

	make_renderable is
			-- Initialize
		do
			use_display_list := False
			expire
		end

feature -- Status report

	use_display_list: BOOLEAN
			-- Enable display list compilation?

	needs_update: BOOLEAN
			-- Expired / needs internal state updated?

feature -- Status setting

	enable_display_list is
			-- Enable display list compilation (might provide speedup)
		do
			use_display_list := True
			expire
		ensure
			enabled: use_display_list
			expired: needs_update
		end

	disable_display_list is
			-- Disable display list compilation (advised if object changes often)
		do
			use_display_list := False
			expire
		ensure
			disabled: not use_display_list
			expired: needs_update
		end

feature -- Update mechanism

	deep_expire, expire is
			-- Expire internal state
		do
			needs_update := True
			valid_display_list := False
		end

	update is
			-- Update internal state
		do
			needs_update := False
		end

feature {EM_VIZ_RENDERABLE} -- Manufacturing

	last_label: EM3D_STRING
			-- Last label created

	create_label (label_text: STRING) is
			-- Make new label with `label_text'
		require
			label_text_exists: label_text /= Void
		do
			create last_label.make_with_format (
				label_text,
				Viz_options.label_font_format
			)
		ensure
			result_exists: last_label /= Void
		end

feature -- Rendering

	render (viewing_direction: EM_VECTOR3D) is
			-- Conditional update / display list compilation and rendering
		do
			conditional_update

			if use_display_list then
				if not valid_display_list then
					vz_compile (viewing_direction)
				end
				display_list.call_list
			else
				vz_setup
				vz_render (viewing_direction)
				vz_revert
			end
		ensure
			valid_display_list: use_display_list implies valid_display_list
		end

feature {EM_VIZ_RENDERABLE} -- Rendering

	render_colored_cube (pmin, pmax: EM_VECTOR3D; a_color: EM_COLOR)
			-- Draw axis-aligned cube with corners `pmin' and `pmax' and `a_color'
		local
			e000, e100, e010, e110, e001, e101, e011, e111: like pmin
		do
			-- Oh, no. Absolutely not, sir. I'm afraid this is the shortest we have.
			e000 := pmin
			e100 := [pmax.x, pmin.y, pmin.z]
			e010 := [pmin.x, pmax.y, pmin.z]
			e001 := [pmin.x, pmin.y, pmax.z]
			e111 := pmax
			e011 := [pmin.x, pmax.y, pmax.z]
			e101 := [pmax.x, pmin.y, pmax.z]
			e110 := [pmax.x, pmax.y, pmin.z]

			vz_color (a_color)
			vz_begin_quads
				-- Front
				vz_normal3d (0, 0, -1)
				vz_vertex (e000)
				vz_vertex (e100)
				vz_vertex (e110)
				vz_vertex (e010)
				-- Back
				vz_normal3d (0, 0, 1)
				vz_vertex (e001)
				vz_vertex (e011)
				vz_vertex (e111)
				vz_vertex (e101)
				-- Left
				vz_normal3d (-1, 0, 0)
				vz_vertex (e000)
				vz_vertex (e010)
				vz_vertex (e011)
				vz_vertex (e001)
				-- Right
				vz_normal3d (1, 0, 0)
				vz_vertex (e100)
				vz_vertex (e101)
				vz_vertex (e111)
				vz_vertex (e110)
				-- Top
				vz_normal3d (0, -1, 0)
				vz_vertex (e000)
				vz_vertex (e001)
				vz_vertex (e101)
				vz_vertex (e100)
				-- Bottom
				vz_normal3d (0, 1, 0)
				vz_vertex (e010)
				vz_vertex (e110)
				vz_vertex (e111)
				vz_vertex (e011)
			vz_end
		end

	render_bounding_box (pmin, pmax: EM_VECTOR3D) is
			-- Draw axis-aligned bounding box with corners `pmin' and `pmax' and
			-- `a_color'
		local
			e000, e100, e010, e110, e001, e101, e011, e111: like pmin
		do
			e000 := pmin
			e100 := [pmax.x, pmin.y, pmin.z]
			e010 := [pmin.x, pmax.y, pmin.z]
			e001 := [pmin.x, pmin.y, pmax.z]
			e111 := pmax
			e011 := [pmin.x, pmax.y, pmax.z]
			e101 := [pmax.x, pmin.y, pmax.z]
			e110 := [pmax.x, pmax.y, pmin.z]

			vz_set_line_width (Viz_options.bounding_box_line_width)
			vz_color (Viz_options.bounding_box_color)
			vz_normal3d (0, 0, -1)

			vz_begin_line_loop
				-- Front
				vz_vertex (e000)
				vz_vertex (e100)
				vz_vertex (e110)
				vz_vertex (e010)
			vz_end
			vz_begin_line_loop
				-- Back
				vz_vertex (e001)
				vz_vertex (e011)
				vz_vertex (e111)
				vz_vertex (e101)
			vz_end
			vz_begin_lines
				-- Edges from front to back
				vz_vertex (e000)
				vz_vertex (e001)
				vz_vertex (e100)
				vz_vertex (e101)
				vz_vertex (e010)
				vz_vertex (e011)
				vz_vertex (e110)
				vz_vertex (e111)
			vz_end
		end

	render_spot (p: EM_VECTOR3D) is
			-- Draw a single point at position `p'
		do
			vz_set_point_size (Viz_options.spot_size)

			vz_begin_points
				vz_normal3d (0, 0, -1)
				vz_color (Viz_options.spot_color)
				vz_vertex (p)
			vz_end
		end

	render_colored_arrow (p, d: EM_VECTOR3D; color: EM_COLOR) is
			-- Render arrow of size `d'.length pointing at `p' in direction of `d'
		require
			color_exists: color /= Void
		local
			n: like d
		do
			n := normal_xy (d).scaled (0.5)

			-- Set color and normal
			vz_color (color)
			vz_normal3d (0, 0, -1)

			-- Arrow
			vz_begin_triangles
				vz_vertex (p + d)
				vz_vertex (p + n)
				vz_vertex (p - n)
			vz_end
		end

	render_axis (s, e: EM_VECTOR3D; tail_arrow, head_arrow: BOOLEAN; blips: DS_LINEAR [DOUBLE]) is
			-- Render axis from `s' to `e', with `has_head',`has_tail', and `blips'
		local
			d, n, p: EM_VECTOR3D
		do
			d := e - s
			if d.length > Epsilon then
				d.normalize
			end

			n := normal_xy (d.scaled (0.5 * Viz_options.axis_blip_size))
			d.scale (Viz_options.axis_arrow_size)

			if head_arrow then
				render_colored_arrow (e, d, Viz_options.axis_color)
			end

			if tail_arrow then
				render_colored_arrow (s, -d, Viz_options.axis_color)
			end

			vz_color (Viz_options.axis_color)
			vz_normal3d (0,0,-1)
			vz_set_line_width (Viz_options.axis_line_width)
			vz_begin_lines
				-- Axis
				vz_vertex (s)
				vz_vertex (e)

				-- Blips
				from
					blips.start
				until
					blips.off
				loop
					p := s + (e - s) * blips.item_for_iteration
					vz_vertex (p + n)
					vz_vertex (p - n)
					blips.forth
				end
			vz_end
		end

	render_link (s, e: EM_VECTOR3D; source_arrow, sink_arrow: BOOLEAN) is
			-- Render a link from `s' to `e'
		local
			sz: DOUBLE
			d: like s
		do
			sz := Viz_options.link_arrow_size
			d := e - s
			if d.length > Epsilon then
				d.normalize
			end
			d.scale (sz)

			if source_arrow then
				render_colored_arrow (s + d, -d, Viz_options.link_source_color)
			end

			if sink_arrow then
				render_colored_arrow (e - d, d, Viz_options.link_sink_color)
			end

			-- Set color and normal
			vz_color (Viz_options.link_sink_color)
			vz_normal3d (0, 0, -1)

			vz_set_line_width (Viz_options.link_line_width)

			vz_begin_lines
				vz_normal3d (0, 0, -1)
				vz_color (Viz_options.link_source_color)
				vz_vertex (s)
				vz_color (Viz_options.link_sink_color)
				vz_vertex (e)
			vz_end
		end

	render_label (p: EM_VECTOR3D; label: EM3D_STRING) is
			-- Draw `label' at position `p'
		do
			label.render_aligned (p, Viz_options.label_alignment)
		end

	render_linked_label (s, e: EM_VECTOR3D; label: EM3D_STRING) is
			-- Draw label at position `s' with link pointing to `e'
		do
			render_link (s, e, False, False)
			label.render_aligned (s, label_alignment_from_pointer (e - s))
		end

	render_box_label (box: EM_INTERVAL_3D; label: EM3D_STRING) is
			-- Draw `label' inside `box'
		local
			pos: EM_VECTOR3D
		do
			pos := box.align_value (box.center, Viz_options.label_box_alignment)
			label.render_aligned (pos, Viz_options.label_box_alignment.to_2d)
		end

	render_grid_lines (step_count: INTEGER; p1, p2, step: EM_VECTOR3D) is
			-- Draw grid-lines, starting with line from `p1' to `p2'
			-- and adding `step' at each iteration, for a total of `step_count' steps.
		local
			i: INTEGER
			lp1, lp2: EM_VECTOR3D
		do
			vz_set_line_width (Viz_options.grid_line_width)

			vz_begin_lines
				from
					lp1 := p1
					lp2 := p2
					i := step_count
				until
					i <= 0
				loop
					vz_color (Viz_options.grid_color)
					vz_vertex (lp1)
					vz_vertex (lp2)

					lp1 := lp1 + step
					lp2 := lp2 + step
				end
			vz_end
		end

feature {EM_VIZ_RENDERABLE} -- Rendering (internal)

	vz_compile (viewing_direction: EM_VECTOR3D) is
			-- Compile display list
		require
			properly_updated: not needs_update
		do
			vz_cache
			if use_display_list and not valid_display_list then
				if display_list = Void then create display_list end
				display_list.new_list_compile
					vz_setup
					vz_render (viewing_direction)
					vz_revert
				display_list.end_list
				valid_display_list := True
			end
		ensure
			updated_if_needed: use_display_list implies valid_display_list
		end

	vz_conditional_compile (viewing_direction: EM_VECTOR3D) is
			-- Only compile if neccessary
		do
			-- if not (use_display_list and valid_display_list) then
			if not use_display_list or not valid_display_list then
				vz_compile (viewing_direction)
			end
		end

	vz_cache is
			-- Nothing to cache by default
		require
			updated_and_ready_to_cache: not needs_update
		do
		end

	vz_setup is
			-- Setup Viz (OpenGL) state
		require
			properly_updated: not needs_update
		do
		ensure
			no_opengl_error: emgl_get_error = 0
		end

	vz_revert is
			-- Revert Viz (OpenGL) state
		require
			properly_updated: not needs_update
		do
		ensure
			no_opengl_error: emgl_get_error = 0
		end

	vz_render (viewing_direction: EM_VECTOR3D) is
			-- Render current using Vz (OpenGL) commands
		require
			properly_updated: not needs_update
		deferred
		ensure
			no_opengl_error: emgl_get_error = 0
		end

feature {NONE} -- Implementation


	normal_xy (v: EM_VECTOR3D): EM_VECTOR3D is
			-- Make orthogonal vector to `v' that lies in the z=0 plane
		do
			if v.to_vector2d.length > Epsilon then
				Result.set (-v.y, v.x, 0.0)
			else
				Result.set (0.0, v.z, 0.0)
			end
		end

	valid_display_list: BOOLEAN
			-- Is display list valid?

	display_list: EMGL_DISPLAY_LIST
			-- Display list

	label_alignment_from_pointer (pointer: EM_VECTOR2D): EM_ALIGNMENT_2D is
			-- Find alignment for linked label
		local
			ndir: like pointer
			th: DOUBLE
		do
			Result.set (Align_center, Align_center)
			if pointer.length /= 0.0 then
				ndir := pointer.normalized
				th := sine (Pi/8)

				if ndir.x < -th then
					Result.set_x (Align_left)
				elseif ndir.x > th then
					Result.set_x (Align_right)
				end
				if ndir.y < -th then
					Result.set_y (Align_top)
				elseif ndir.y > th then
					Result.set_y (Align_bottom)
				end
			end
		end

end
